//----< depMatrix.h > ---------------------------------------------
//---- Author: Qiang Zheng ------------------------------------------
//
//---- Purpose: To use an adjecent matrix to express the file dependecies
//----          And to maintain this matrix.
//----          And an assotiative array is used to associate the filename
//----          and the index of the row of the matrix.
//
//---- e.g. file1.cpp-->file2.h--> 
//            |                  |
//  	       -------->file3.h---->file4.h  (Here "-->" means "is dependent on")
//
//---- The corresponding adjecent matrix is:
//---- (T,T,T,T
//----  F,T,F,T
//----  F,F,F,T
//----  F,F,F,T)
//
//---- In this array, say dependence, dependence[i][j] = T,<==> file i depends on file j, 
//---- Attention: A file is defined to be dependent on itself
//
//---- And in the associative array fileNames[], fileNames[0] = "file1.cpp",
//---- fileNames[1] = "file2.h"; fileNames[2] = "file3.h", fileName[3] = "file4.h";
#ifndef __DEP_MATRIX_INCLUDED
#define __DEP_MATRIX_INCLUDED

#include <string.h>	//for strlen, strcpy

inline char *newStr( const char *s )
{
	char *tem = new char[strlen(s)+1];
	strcpy( tem, s );
	return tem;
}

class depMatrix 
{
public:
	// constructors and destructor
	depMatrix();
	~depMatrix();

	//return the number of the files in the matrix
	int getSize();

	//return the index of a file in the dependence matrix
	//If the file already exists in matrix set it returns index of current file, 
	// or add it as a new node in matrix
	//It also return index of the file;
	int indexOf( const char * );

	void setDependence( int src, int dst );

	void outputDependence();
protected:
	int size;		//the size of the matrix is (size * size);

	bool *dependence;	//the adjecent matrix to represent the dependece relation
	char **fileNames;
};

#endif //__DEP_MATRIX_INCLUDED